# Analyzing Phylogenetic Signal in Phonotactics of Pama-Nyungan Languages
# Jayden Macklin-Cordes

##############################
# SET UP
##############################

# Load packages
library(tidyverse)
library(ape)
library(caper)
library(reshape2)
library(ggplot2)
library(e1071)
library(picante)
library(rcompanion)
library(phylobase)
library(phylosignal)
library(kSamples)

# Set seed for reproducibility of stochastic processes
set.seed(2019)

# Read reference tree
PN_SDollo_sum <- read.nexus("../trees/PNY10_285.(time).sum.tree")

# Read modified version of functions from Caper package
source("modified_caper_funcs.R")

##############################
# PREPARE DATA
##############################

# Function to filter out characters with too many missing values
# The rm_0 parameter optionally converts 0 values to NAs (important for K test)
filter_phylo_data <- function(dataset, lim = 50, rm_0 = FALSE) {
  if (rm_0 == TRUE) {
    dataset[dataset == 0] <- NA
  }
  df <- select_if(dataset, ~sum(!is.na(.)) >= lim)
  df
}

# Function to drop uninformative variables (where all values are the same)
drop_uninformative_vars <- function(dataset) {
  uniquelength <- sapply(dataset, function(x) length(unique(x[!is.na(x)])))
  df <- subset(dataset, select = uniquelength > 1)
  df
}

# Prepare binary data (min 50 non-NA values)
binary_data <- read_csv("../data/biphone_binary_2020-08-07.csv") %>%
  filter_phylo_data %>%
  drop_uninformative_vars

# Prepare biphone frequency data (min 20 non-NA values)
K_biphones_fwd  <- read_csv("../data/biphone_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_biphones_bkwd <- read_csv("../data/biphone_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

# Prepare sound class frequency data (min 20 non-NA values)
K_place_fwd <- read_csv("../data/place_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_place_bkwd <- read_csv("../data/place_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_major_place_fwd <- read_csv("../data/major_place_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_major_place_bkwd <- read_csv("../data/major_place_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_manner_fwd       <- read_csv("../data/manner_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_manner_bkwd      <- read_csv("../data/manner_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

##############################
# D TEST
##############################

# (run time: ~30 min on 8GB Macbook Pro)
create_comparative_data_object <- function(phy = PN_SDollo_sum, dat = binary_data, chars, tip_names = "tip_label") {
  df <- dplyr::select(dat, tip_label, one_of(chars))
  df <- as.data.frame(df)
  rownames(df) <- df$tip_labels
  
  mod.comparative.data(phy, df, names.col = tip_names)
}

d_test <- function(phy = PN_SDollo_sum, dat = binary_data, char, tip_names = "tip_label", permut = 1000) {
  cd <- create_comparative_data_object(phy = phy, dat = dat, chars = char, tip_names = tip_names)
  d <- mod.phylo.d(cd, binvar = char, permut = permut)
  d
}

multi_d_test <- function(phy = PN_SDollo_sum, dat = binary_data, chars, tip_names = "tip_label", permut = 1000) {
  base::lapply(chars, function(x) d_test(phy = phy, dat = dat, char = x, tip_names = tip_names, permut = permut))
}

# Analyse all variables and tabulate results
full_d_test <- multi_d_test(chars = names(binary_data)[-1], permut = 10000) # produces large list (415 elements, ~520Mb)

d_test_table <- tibble(biphone      = as.character(unlist(sapply(full_d_test, `[`, "binvar"))),
                       languages    = sapply(full_d_test, function(x) sum(x$StatesTable)),
                       count_0      = as.numeric(unlist(sapply(full_d_test, function(x) x$StatesTable[1]))),
                       count_1      = as.numeric(unlist(sapply(full_d_test, function(x) x$StatesTable[2]))),
                       D            = as.numeric(unlist(sapply(full_d_test, `[`, "DEstimate"))),
                       Pval_0       = as.numeric(unlist(sapply(full_d_test, `[`, "Pval0"))),
                       Pval_1       = as.numeric(unlist(sapply(full_d_test, `[`, "Pval1")))
)

two_tailed_Pval0 <- function(Pval_0, alpha = 0.05) {
  if (Pval_0 > (1 - alpha / 2)) {
    sig <- "more clumped than Brownian expectation"
  } else if (Pval_0 < (alpha / 2)) {
    sig <- "more dispersed than Brownian expectation"
  } else {
    sig <- "consistent with Brownian expectation"
  }
  sig
}

two_tailed_Pval1 <- function(Pval_1, alpha = 0.05) {
  if (Pval_1 > (1 - alpha / 2)) {
    sig <- "more dispersed than random expectation"
  } else if (Pval_1 < (alpha / 2)) {
    sig <- "more clumped than random expectation"
  } else {
    sig <- "consistent with random expectation"
  }
  sig
}

d_test_table$Pval0_sig <- sapply(d_test_table$biphone, function(x) two_tailed_Pval0(d_test_table[d_test_table$biphone == x, "Pval_0"], alpha = 0.025))
d_test_table$Pval1_sig <- sapply(d_test_table$biphone, function(x) two_tailed_Pval1(d_test_table[d_test_table$biphone == x, "Pval_1"], alpha = 0.025))

describe_significance <- function (Pval0_sig, Pval1_sig) {
  if (Pval0_sig == "more clumped than Brownian expectation" &
      Pval1_sig == "more clumped than random expectation") {
    sig <- "More clumped"
  } else if (Pval0_sig == "more dispersed than Brownian expectation" &
             Pval1_sig == "more dispersed than random expectation") {
    sig <- "More dispersed"
  } else if (Pval0_sig == "consistent with Brownian expectation" &
             Pval1_sig == "more clumped than random expectation") {
    sig <- "Phylogenetic"
  } else if (Pval0_sig == "more dispersed than Brownian expectation" &
             Pval1_sig == "consistent with random expectation") {
    sig <- "Random"
  } else if (Pval0_sig == "more dispersed than Brownian expectation" &
             Pval1_sig == "more clumped than random expectation") {
    sig <- "0 < D < 1 (both H0s rejected)"
  } else if (Pval0_sig == "consistent with Brownian expectation" &
             Pval1_sig == "consistent with random expectation") {
    sig <- "Indeterminate (neither H0 rejected)"
  } else if (Pval0_sig == "more clumped than Brownian expectation" &
             Pval1_sig == "consistent with random expectation") {
    sig <- "Indeterminate (neither H0 rejected)"
  }
  sig
}

d_test_table$result <- as.character(lapply(d_test_table$biphone, function(x) describe_significance(d_test_table[d_test_table$biphone == x, "Pval0_sig"], d_test_table[d_test_table$biphone == x, "Pval1_sig"])))

d_test_table$Pval0_sig <- as.factor(d_test_table$Pval0_sig)
d_test_table$Pval1_sig <- as.factor(d_test_table$Pval1_sig)
d_test_table$result    <- as.factor(d_test_table$result)

write_csv(d_test_table, paste0("../results/D_test_results_", Sys.Date(), ".csv"))

# To get some summary statistics, run the following:
# summary(d_test_table)
# d_test_table <- read_csv("../results/D_test_results_2020-08-07.csv")

# Five moments of the distribution of D test values
# moments <- function(dat) {
#   cat(paste0(
#     "Mean: ", mean(dat), "\n",
#     "Std dev: ", sd(dat), "\n",
#     "Median: ", median(dat), "\n",
#     "Skewness: ", skewness(dat), "\n",
#     "Kurtosis: ", kurtosis(dat), "\n"
#   )
#   )
# }
# 
# moments(d_test_table$D)

##############################
# K TEST WITH BIPHONES
##############################

# Create data frame with rownames, as required by multiPhylosignal
create_char_df <- function(dat) {
  df <- as.data.frame(dat)
  rownames(df) <- df$tip_label
  df[, "tip_label"] <- NULL
  df
}

# Extra functions for tabulating results
one_tailed_p <- function (p, alpha = 0.05) {
  if (p < alpha) {
    sig <- "yes"
  } else {
    sig <- "no"
  }
  sig
}

extra_K_analysis <- function(results, dataset) {
  results$significant <- sapply(results$PIC.variance.P, function(x) as.factor(one_tailed_p(x)))
  results$biphone <- rownames(results)
  results$n_langs <- sapply(results$biphone, function(x) sum(!is.na(dataset[,x])))
  results
}

# Run test
# (run time: ~30 min on 8GB Macbook Pro)
K_biphones_fwd_results  <- K_biphones_fwd %>% 
  create_char_df %>% 
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_biphones_fwd)
  
K_biphones_bkwd_results <- K_biphones_bkwd %>% 
  create_char_df %>% 
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_biphones_bkwd)

write_csv(K_biphones_fwd_results, paste0("../results/K_biphones_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_biphones_bkwd_results, paste0("../results/K_biphones_bkwd_results_", Sys.Date(), ".csv"))

# To inspect results, run the following:
# K_biphones_fwd_results <- read_csv("../results/K_biphones_fwd_results_2020-08-07.csv")
# K_biphones_fwd_results$significant <- as.factor(K_biphones_fwd_results$significant)
# K_biphones_bkwd_results <- read_csv("../results/K_biphones_bkwd_results_2020-08-07.csv")
# K_biphones_bkwd_results$significant <- as.factor(K_biphones_bkwd_results$significant)

# There is no statistically significant difference between K tests for 
# forward transition frequences and backward transition frequencies:
# fwd <- dplyr::select(K_biphones_fwd_results, K)
# bkwd <- dplyr::select(K_biphones_bkwd_results, K)
# t.test(fwd$K, bkwd$K)
# ks.test(fwd$K, bkwd$K)

# Plot the distribution of values for every character in the dataset.
# This produces a large sheet of density plots, one for each character: 
# library(purrr)
# 
# biphones_fwd %>%
#   keep(is.numeric) %>% 
#   gather() %>% 
#   ggplot(aes(value)) +
#   facet_wrap(~ key, scales = "free") +
#   geom_density(fill = "grey") +
#   theme_classic()
# 
# biphones_bkwd %>%
#   keep(is.numeric) %>%
#   gather() %>%
#   ggplot(aes(value)) +
#   facet_wrap(~ key, scales = "free") +
#   geom_density(fill = "grey") +
#   theme_classic()

# Plot the same as above, but with normalised character values:
# library(purrr)
# 
# biphones_fwd_norm %>%
#   keep(is.numeric) %>% 
#   gather() %>% 
#   ggplot(aes(value)) +
#   facet_wrap(~ key, scales = "free") +
#   geom_density(fill = "grey") +
#   theme_classic()
# 
# biphones_bkwd_norm %>%
#   keep(is.numeric) %>%
#   gather() %>%
#   ggplot(aes(value)) +
#   facet_wrap(~ key, scales = "free") +
#   geom_density(fill = "grey") +
#   theme_classic()

# Rerunning the same $K$ analysis as before, but with normalized character values:

K_biphones_fwd_norm_results <- K_biphones_fwd %>%
  create_char_df %>%
  apply(2, function(x) transformTukey(x, plotit = FALSE)) %>% 
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_biphones_fwd)
K_biphones_bkwd_norm_results <- K_biphones_bkwd %>%
  create_char_df %>%
  apply(2, function(x) transformTukey(x, plotit = FALSE)) %>% 
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_biphones_bkwd)

write_csv(K_biphones_fwd_norm_results, paste0("../results/K_biphones_fwd_norm_results_", Sys.Date(), ".csv"))
write_csv(K_biphones_bkwd_norm_results, paste0("../results/K_biphones_bkwd_norm_results_", Sys.Date(), ".csv"))

# To inspect results, run the following:
# K_biphones_fwd_norm_results <- read_csv("../results/K_biphones_fwd_norm_results_2020-08-07.csv")
# K_biphones_fwd_norm_results$significant <- as.factor(K_biphones_fwd_norm_results$significant)
# K_biphones_bkwd_norm_results <- read_csv("../results/K_biphones_bkwd_norm_results_2020-08-07.csv")
# K_biphones_bkwd_norm_results$significant <- as.factor(K_biphones_bkwd_norm_results$significant)

# Compare K analyses with untransformed character values versus normalised values:
# raw <- bind_rows(K_biphones_fwd_results, K_biphones_bkwd_results) %>%
# dplyr::select(K)
# norm <- bind_rows(K_biphones_fwd_norm_results, K_biphones_bkwd_norm_results) %>%
#   dplyr::select(K)

# There is no statistically significant difference between means of $K$:
# t.test(raw$K, norm$K)
# ks.test(raw$K, norm$K)

##############################
# K TEST WITH NATURAL CLASSES
##############################

# Run test
K_place_fwd_results <- K_place_fwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_place_fwd)
K_place_bkwd_results <- K_place_bkwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_place_bkwd)
K_major_place_fwd_results <- K_major_place_fwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_major_place_fwd)
K_major_place_bkwd_results <- K_major_place_bkwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_major_place_bkwd)
K_manner_fwd_results <- K_manner_fwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_manner_fwd)
K_manner_bkwd_results <- K_manner_bkwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_manner_bkwd)

write_csv(K_place_fwd_results, paste0("../results/K_place_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_place_bkwd_results, paste0("../results/K_place_bkwd_results_", Sys.Date(), ".csv"))
write_csv(K_major_place_fwd_results, paste0("../results/K_major_place_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_major_place_bkwd_results, paste0("../results/K_major_place_bkwd_results_", Sys.Date(), ".csv"))
write_csv(K_manner_fwd_results, paste0("../results/K_manner_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_manner_bkwd_results, paste0("../results/K_manner_bkwd_results_", Sys.Date(), ".csv"))

# To inspect results, run the following:
# K_place_fwd_results        <- read_csv("../results/K_place_fwd_results_2020-08-07.csv")
# K_place_bkwd_results       <- read_csv("../results/K_place_bkwd_results_2020-08-07.csv")
# K_major_place_fwd_results  <- read_csv("../results/K_major_place_fwd_results_2020-08-07.csv")
# K_major_place_bkwd_results <- read_csv("../results/K_major_place_bkwd_results_2020-08-07.csv")
# K_manner_fwd_results       <- read_csv("../results/K_manner_fwd_results_2020-08-07.csv")
# K_manner_bkwd_results      <- read_csv("../results/K_manner_bkwd_results_2020-08-07.csv")

# Compare all natural class types:
# all_features <- bind_rows(K_place_fwd_results, K_place_bkwd_results,
#                           K_major_place_fwd_results, K_major_place_bkwd_results,
#                           K_manner_fwd_results, K_manner_bkwd_results)
# all_features$significant <- as.factor(all_features$significant)
# 
# summary(all_features)

# Compare K for biphone characters versus natural class-based characters:
# all_biphones <- bind_rows(K_biphones_fwd_results, K_biphones_bkwd_results) %>%
#   dplyr::select(K)
# all_features <- dplyr::select(all_features, K)

# Forward and backward transition probabilities for transitions between natural classes show a greater degree of phylogenetic signal than biphone-based characters:
# t.test(all_biphones$K, all_features$K)
# ks.test(all_biphones$K, all_features$K)

# Compare K for different natural class types:
# all_place <- bind_rows(K_place_fwd_results, K_place_bkwd_results) %>%
#   dplyr::select(K)
# all_place$class_type <- "Place features"
# all_major_place <- bind_rows(K_major_place_fwd_results, K_major_place_bkwd_results) %>%
#   dplyr::select(K)
# all_major_place$class_type <- "Major place features"
# all_manner <- bind_rows(K_manner_fwd_results, K_manner_bkwd_results) %>%
#   dplyr::select(K)
# all_manner$class_type <- "Manner features"

# The difference in means between the three different kinds of natural classes is not significant according to a one-way ANOVA:
# k_all_classes <- bind_rows(all_place, all_major_place, all_manner)
# 
# fit = lm(K ~ class_type, k_all_classes)
# anova(fit)
# ad.test(K ~ class_type, data = k_all_classes)

##############################
# BROWNIAN MOTION SIMULATION
##############################

# Simulation of Brownian motion on the Pama-Nyungan phylogeny with various levels of noise
# PN_SDollo_sum_4 <- as(PN_SDollo_sum, "phylo4")
# sim <- phyloSim(PN_SDollo_sum_4, methods = "K", nsim = 100, reps = 100, pb = FALSE)

# Inspect results:
# plot(sim, quantiles = c(0.05, 0.95))
# plot(sim, what = "pval")
