################################################################################
# CREATE FIGURES
################################################################################

library(tidyverse)
library(ggplot2)
library(reshape2)
library(extrafont)
library(cowplot)
library(ape)
library(phylobase)
library(phylosignal)
library(kableExtra)
library(gridGraphics)

set.seed(2019)
loadfonts(device = "pdf", quiet = TRUE)

if(dir.exists("../fig")==FALSE) dir.create("../fig")

################################################################################
# Wordlist sizes
# Get wordlist details
wordlists <- read_csv("../data/wordlist_sizes_2020-08-07.csv")

wordlist_sizes <- ggplot(wordlists,
                         aes(x = reorder(tip_label, -n_entries), y = n_entries)) +
  geom_col(fill = "tomato3") +
  xlab("Language variety") +
  ylab("Wordlist size") +
  scale_y_continuous(expand = c(0, 0), limits = c(-25,5150)) +
  theme_classic(base_size = 10) +
  theme(plot.margin = margin(10,2,2,10,"pt"),
        axis.text.x = element_text(family = "Noto Sans",
                                   angle = 90, hjust=1, vjust=0.5, size = 6))

cairo_pdf("../fig/wordlist_sizes.pdf", width = 10, height = 4)
wordlist_sizes
dev.off()

################################################################################
# D test density plot
# Exported to fig/d-density.pdf

d_test_table <- read_csv("../results/D_test_results_2020-08-12.csv")

d_density_plot <- ggplot(d_test_table, aes(D)) +
  geom_density(fill = "grey80") +
  geom_vline(aes(xintercept = mean(d_test_table$D)), colour = "red") +
  geom_vline(aes(xintercept = 0), linetype = "dashed") +
  geom_vline(aes(xintercept = 1), linetype = "dashed") +
  scale_x_continuous(breaks = c(-10, 0, 1, 10, 20)) +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/d-density.pdf", width = 6.472, height = 4)
d_density_plot
dev.off()

################################################################################
# D test swatch plot
# Exported to fig/d-sig-swatch.pdf

# IPA order function from Ausphonlex
ausphon_IPA_order <- function () {
  c("i", "ĩ", "y", "æ", "æ̃", "e", "ẽ", "ɛ", "ə", "ɨ",
    "ɵ", "ʉ", "a", "ã", "u", "o", "õ", "ɔ", "iː", "æː", "eː",
    "ø", "øː", "ɛː", "əː", "ɨː", "ɵː", "ʉː", "aː",
    "uː", "oː", "ɔː", "break",
    "#", "~", "?", ";", "⁇", "!", "=", "+",
    "-", ".", "|", "break",
    "t̺͈", "t͈", "ʈ͈", "t̻͈", "\u0236͈", "t̪͈",
    "k͈", "p͈", "t̺", "t", "ʈ", "t̻", "\u0236", "t̪", "k",
    "p", "ʔ", "t̺͉", "t͉", "ʈ͉", "t̻͉", "\u0236͉", "t̪͉",
    "k͉", "p͉", "s̺", "ʂ", "s̻", "ɕ", "θ", "x", "ɸ", "h",
    "z̺", "ʐ", "z̻", "ʑ", "ð", "ɣ", "β", "ɦ", "break",
    "n̺͈", "n͈",
    "ɳ͈", "n̻͈", "\u0235͈", "n̪͈", "ŋ͈", "m͈", "n̺", "n",
    "ɳ", "n̻", "\u0235", "n̪", "ŋ", "m", "break",
    "r̺̥", "r̥", "r̺",
    "\u024d̺", "r", "\u024d", "ɾ̺", "ɾ", "ɽ", "l̺͈", "l͈",
    "ɭ͈", "l̻͈", "\u0234͈", "l̪͈", "l̺", "l", "ɭ", "l̻",
    "\u0234", "l̪", "ɺ̺", "ɺ", "ɺ̢", "ɹ̺", "ɻ", "j̻", "j",
    "j̪", "ɰ", "w")
}

# Gather D test results
d_test_table <- read_csv("../results/D_test_results_2020-08-12.csv")
d_results <- dplyr::select(d_test_table, value = result)
d_results[,2:3] <- colsplit(d_test_table$biphone, "_", names = c("phone1", "phone2"))
d_results_matrix <- as.data.frame(spread(d_results, phone2, value))
rownames(d_results_matrix) <- d_results_matrix$phone1
d_results_matrix <- d_results_matrix[,-1]

# Get data df into shape
df <- as.data.frame(d_results_matrix)
df$phone1 <- rownames(df)
df <- gather(df, phone2, value, -phone1)
df$value <- as.factor(df$value)
df$value <- factor(df$value, levels(df$value)[c(3,4,1,5,2)])

# Establish the phones & order to plot
phones1 <- unique(df$phone1)
phones2 <- unique(df$phone2)
phones <- union(phones1, phones2)

order <- ausphon_IPA_order()
is_break <- str_to_lower(order) == "break"
order[is_break] <- ""

plot_length1 <- sum(is_break) + length(phones1)
plot_length2 <- sum(is_break) + length(phones2)
order1 <- order[order %in% c("", phones1)]
order2 <- order[order %in% c("", phones2)]
df$phone1_order <- match(df$phone1, order1)
df$phone2_order <- match(df$phone2, order2)

# cols based on 4-class RdBu diverging scale @ colorbrewer2.org
cols = c("More clumped" = "#f4a582",
         "Phylogenetic" = "#ca0020",
         "0 < D < 1 (both H0s rejected)" = "#683868",
         "Random" = "#0571b0",
         "More dispersed" = "#92c5de",
         "Indeterminate (neither H0 rejected)" = "grey80"
         )

# Plot
d_swatch <- ggplot() +
  scale_x_continuous(breaks = 1:plot_length2 - 0.5,
                     labels = order2) +
  scale_y_continuous(breaks = -(1:plot_length1 - 0.5),
                     labels = order1) +
  geom_rect(data = df,
            mapping = aes(xmin = phone2_order - 1,
                          xmax = phone2_order,
                          ymax = -(phone1_order - 1),
                          ymin = -(phone1_order),
                          fill = value
            )
  ) +
  scale_fill_manual(values = cols,
                    breaks = c("More clumped",
                               "Phylogenetic",
                               "Random",
                               "Indeterminate (neither H0 rejected)"),
                    guide = guide_legend(title = NULL)
  ) +
  theme_minimal() +
  theme(panel.grid = element_blank(),
        axis.title = element_blank(),
        axis.text = element_text(family = "Noto Sans",
                                 size = 12),
        plot.margin = margin(.2, .2, .2, .2, "cm"))

# Save file
loadfonts(device = "postscript", quiet = TRUE)
cairo_pdf(file = "../fig/d-sig-swatch.pdf", width = 8.90, height = 5.5)
d_swatch
dev.off()

################################################################################
# D scatterplots

d_test_table$Skewness <- sapply(d_test_table$biphone, function(x)
  max(filter(d_test_table, biphone == x) %>% dplyr::select(count_0, count_1)) /
    sum(filter(d_test_table, biphone == x) %>% dplyr::select(count_0, count_1))
)

d_test_table$result <- as.factor(d_test_table$result)
d_test_table$result <- factor(d_test_table$result, levels(d_test_table$result)[c(2,3,4,1)])

# cols based on 4-class RdBu diverging scale @ colorbrewer2.org
cols = c("More clumped" = "#f4a582",
         "Phylogenetic" = "#ca0020",
         "0 < D < 1 (both H0s rejected)" = "#683868",
         "Random" = "#0571b0",
         "More dispersed" = "#92c5de",
         "Indeterminate (neither H0 rejected)" = "grey80"
         )

col_scale <- scale_fill_manual(values = cols)

nlangs <-   ggplot(d_test_table, aes(D, languages)) + 
  geom_point(aes(fill = result), colour = "black", pch = 21, size = 3, alpha = 0.7) +
  col_scale +
  labs(y = "n language varieties", fill = "Result") +
  theme_cowplot() +
  theme(plot.margin = margin(7,7,7,12))

skewness <- ggplot(d_test_table, aes(D, Skewness)) + 
  geom_point(aes(fill = result), colour = "black", pch = 21, size = 3, alpha = 0.7) +
  col_scale +
  labs(result = "Result") +
  theme_cowplot() +
  theme(plot.margin = margin(7,7,7,12))

legend <- get_legend(
  nlangs +
    theme(legend.position =  "bottom",
          legend.box.margin = margin(0,0,0,12),
          legend.text = element_text(size = 11))
)

prow <- plot_grid(
  nlangs + theme(legend.position = "none"),
  skewness + theme(legend.position = "none"),
  labels = c("(A)", "(B)"),
  hjust = -0.1
)

d_scatterplots <- plot_grid(prow, legend, ncol = 1, rel_heights = c(1, .1))

# Save file
cairo_pdf("../fig/d-scatterplots.pdf", width = 9.708, height = 4.5)
d_scatterplots
dev.off()

################################################################################
# Filtered D density plot

filtered_d_test_table <- filter(d_test_table, Skewness < .97)

d_density_plot_filtered <- ggplot(filtered_d_test_table, aes(D)) +
  geom_density(fill = "grey80") +
  geom_vline(aes(xintercept = mean(d_test_table$D)), colour = "red") +
  geom_vline(aes(xintercept = 0), linetype = "dashed") +
  geom_vline(aes(xintercept = 1), linetype = "dashed") +
  scale_x_continuous(breaks = c(-2, -1, 0, 1, 2)) +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/d-density-filtered.pdf", width = 6.472, height = 4)
d_density_plot_filtered
dev.off()

################################################################################
# Filtered D swatch plot

# Gather D scores
d_scores <- dplyr::select(filtered_d_test_table, D)
d_scores[,2:3] <- colsplit(filtered_d_test_table$biphone, "_", names = c("phone1", "phone2"))
d_matrix <- as.data.frame(spread(d_scores, phone2, D))
rownames(d_matrix) <- d_matrix$phone1
d_matrix <- d_matrix[,-1]

# Get data df into shape
df <- as.data.frame(d_matrix)
df$phone1 <- rownames(df)
df <- gather(df, phone2, value, -phone1)

# Establish the phones & order to plot
phones1 <- unique(df$phone1)
phones2 <- unique(df$phone2)
phones <- union(phones1, phones2)

order <- ausphon_IPA_order()
is_break <- str_to_lower(order) == "break"
order[is_break] <- ""

plot_length1 <- sum(is_break) + length(phones1)
plot_length2 <- sum(is_break) + length(phones2)
order1 <- order[order %in% c("", phones1)]
order2 <- order[order %in% c("", phones2)]
df$phone1_order <- match(df$phone1, order1)
df$phone2_order <- match(df$phone2, order2)

# Plot
d_swatch_filtered <- ggplot() +
  scale_x_continuous(breaks = 1:plot_length2 - 0.5,
                     labels = order2) +
  scale_y_continuous(breaks = -(1:plot_length1 - 0.5),
                     labels = order1) +
  geom_rect(data = df,
            mapping = aes(xmin = phone2_order - 1,
                          xmax = phone2_order,
                          ymax = -(phone1_order - 1),
                          ymin = -(phone1_order),
                          fill = value
            )
  ) +
  scale_fill_gradientn(
  # Based on 10-class RdBu diverging scale @ colorbrewer2.org
  colours = c("white", "#fddbc7", "#b2182b", "#2166ac", "#d1e5f0", "white"),
  values = c(min(df$value, na.rm = TRUE), -1, 0, 1, 2, max(df$value, na.rm = TRUE)),
  na.value = "grey90",
  rescaler = function(x, ...) x,
  guide = guide_colourbar(
    title = "D",
    barheight = unit(0.5, "npc")
    )
  ) +
  theme_minimal() +
  theme(panel.grid = element_blank(),
        axis.title = element_blank(),
        axis.text = element_text(family = "Noto Sans",
                                 size = 12),
        plot.margin = margin(.2, .2, .2, .2, "cm"))

cairo_pdf("../fig/d-swatch-filtered.pdf", width = 8.5, height = 6.5)
d_swatch_filtered
dev.off()

################################################################################
# K TESTS
################################################################################

K_biphones_fwd_results <- read_csv("../results/K_biphones_fwd_results_2020-08-12.csv")
K_biphones_fwd_results$significant <- as.factor(K_biphones_fwd_results$significant)
K_biphones_bkwd_results <- read_csv("../results/K_biphones_bkwd_results_2020-08-12.csv")
K_biphones_bkwd_results$significant <- as.factor(K_biphones_bkwd_results$significant)

K_biphones_fwd_norm_results <- read_csv("../results/K_biphones_fwd_norm_results_2020-08-12.csv")
K_biphones_fwd_norm_results$significant <- as.factor(K_biphones_fwd_norm_results$significant)
K_biphones_bkwd_norm_results <- read_csv("../results/K_biphones_bkwd_norm_results_2020-08-12.csv")
K_biphones_bkwd_norm_results$significant <- as.factor(K_biphones_bkwd_norm_results$significant)

################################################################################
# K density plot

k_all_biphones <- bind_rows(K_biphones_fwd_results, K_biphones_bkwd_results)
k_density <- ggplot(k_all_biphones, aes(K)) +
  geom_density(fill = "grey80") +
  geom_vline(aes(xintercept = mean(k_all_biphones$K)), colour = "red") +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/k-density.pdf", width = 6.472, height = 4)
k_density
dev.off()

################################################################################
# K swatch plot

# Gather K test results
k_results <- dplyr::select(K_biphones_fwd_results, value = K)
k_results[,2:3] <- colsplit(K_biphones_fwd_results$biphone, "_", names = c("phone1", "phone2"))
k_results <- as.data.frame(spread(k_results, phone2, value))
rownames(k_results) <- k_results$phone1
k_results <- k_results[,-1]

# Get data df into shape
df <- as.data.frame(k_results)
df$phone1 <- rownames(df)
df <- gather(df, phone2, value, -phone1)

# Establish the phones & order to plot
phones1 <- unique(df$phone1)
phones2 <- unique(df$phone2)
phones <- union(phones1, phones2)

order <- ausphon_IPA_order()
is_break <- str_to_lower(order) == "break"
order[is_break] <- ""

plot_length1 <- sum(is_break) + length(phones1)
plot_length2 <- sum(is_break) + length(phones2)
order1 <- order[order %in% c("", phones1)]
order2 <- order[order %in% c("", phones2)]
df$phone1_order <- match(df$phone1, order1)
df$phone2_order <- match(df$phone2, order2)

# Plot
k_swatch <- ggplot() +
  scale_x_continuous(breaks = 1:plot_length2 - 0.5,
                     labels = order2) +
  scale_y_continuous(breaks = -(1:plot_length1 - 0.5),
                     labels = order1) +
  geom_rect(data = df,
            mapping = aes(xmin = phone2_order - 1,
                          xmax = phone2_order,
                          ymax = -(phone1_order - 1),
                          ymin = -(phone1_order),
                          fill = value
            )
  ) +
  scale_fill_gradientn(values = seq(0,2,length.out = 19),
# Based on 9-class Reds sequential scale @ colorbrewer2.org
                       colours = c("white", "#fff5f0", "#fee0d2", "#fcbba1", "#fc9272",
                                   "#fb6a4a", "#ef3b2c", "#cb181d", "#a50f15", "#67000d",
                                   "#a50f15", "#cb181d", "#ef3b2c", "#fb6a4a", "#fc9272",
                                   "#fcbba1","#fee0d2","#fff5f0","white"),
                       na.value = "grey90",
                       rescaler = function(x, ...) x,
                       guide = guide_colourbar(
                         title = "K",
                         barheight = unit(0.5, "npc")
    )
                       ) +
  theme_minimal() +
  theme(panel.grid = element_blank(),
        axis.title = element_blank(),
        axis.text = element_text(family = "Noto Sans",
                                 size = 12),
        plot.margin = margin(.2, .2, .2, .2, "cm"))

cairo_pdf("../fig/k-swatch.pdf", width = 8.5, height = 6.5)
k_swatch
dev.off()

################################################################################
# K simulation

PN_SDollo <- read.nexus("../trees/PNY10_285.(time).sum.tree")
PN_SDollo_pruned <- drop.tip(PN_SDollo, setdiff(PN_SDollo$tip.label, wordlists$tip_label))
PN_SDollo_sum_4 <- as(PN_SDollo_pruned, "phylo4")

sim <- phyloSim(PN_SDollo_sum_4, methods = "K", nsim = 1000, reps = 1000, pb = FALSE)

# Calculating Type I error rate
vals <- c(1, 1+1:999*101)
p_rand <- sim$sim.pvalue[vals]
length(p_rand[p_rand < 0.05]) / 1000

# Calculating Type II error rate
vals <- c(1:1000*101)
p_brown <- sim$sim.pvalue[vals]
1 - length(p_brown[p_brown < 0.05]) / 1000

sim_stat <- function () {
  par(xpd = NA, # switch off clipping, necessary to always see axis labels
      bg = "transparent", # switch off background to avoid obscuring adjacent plots
      oma = c(2, 2, 0, 0),
      mar = c(c(3, 4, 4, 2) + 0.1))
  plot.phylosim(sim, quantiles = c(0.05, 0.95))
}

sim_pval <- function () {
  par(xpd = NA, # switch off clipping, necessary to always see axis labels
      bg = "transparent", # switch off background to avoid obscuring adjacent plots
      oma = c(2, 2, 0, 0),
      mar = c(c(3, 4, 4, 2) + 0.1))
  plot(sim, what = "pval")
}

cairo_pdf("../fig/phylosim.pdf", width = 10, height = 4)
plot_grid(sim_stat, sim_pval, labels = c("(A)", "(B)"))
dev.off()

################################################################################
# K scatterplot

K_scatterplot <- ggplot(bind_rows(K_biphones_fwd_results, K_biphones_bkwd_results), aes(K, n_langs)) + 
  geom_point(aes(fill = significant), colour = "black", pch = 21, size = 2, alpha = 0.7) +
  scale_fill_manual(values = c("yes" = "#d7191c", "no" = "#2c7bb6")) +
  labs(y = "n language varieties", fill = "Significant") +
  theme_cowplot()

cairo_pdf("../fig/k-scatterplot.pdf", width = 6.472, height = 4)
K_scatterplot
dev.off()

################################################################################
# Forward vs backward transitions

fwd <- dplyr::select(K_biphones_fwd_results, K)
fwd$Direction <- "Forward"
bkwd <- dplyr::select(K_biphones_bkwd_results, K)
bkwd$Direction <- "Backward"

df <- bind_rows(fwd, bkwd)

fwd_vs_bkwd <- ggplot(df, aes(K, fill = Direction)) +
  scale_fill_manual(values = c("Forward" = "#d7191c", "Backward" = "#2c7bb6")) +
  geom_density(alpha = 0.5) +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/k-fwd-bkwd.pdf", width = 6.472, height = 4)
fwd_vs_bkwd
dev.off()

################################################################################
# Original vs normalised data values

orig <- bind_rows(K_biphones_fwd_results, K_biphones_bkwd_results) %>%
  dplyr::select(K)
orig$transformation <- "None"
norm <- bind_rows(K_biphones_fwd_norm_results, K_biphones_bkwd_norm_results) %>%
  dplyr::select(K)
norm$transformation <- "Normal"

df <- bind_rows(orig, norm)

orig_vs_norm <- ggplot(df, aes(K, fill = transformation)) +
  geom_density(alpha = 0.5) +
  scale_fill_manual(values = c("Normal" = "#d7191c", "None" = "#2c7bb6")) +
  labs(y = "Density", fill = "Transformation") +
  theme_cowplot()

cairo_pdf("../fig/k-orig-norm.pdf", width = 6.472, height = 4)
orig_vs_norm
dev.off()

################################################################################
# K TESTS WITH NATURAL SOUND CLASSES
################################################################################

K_place_fwd_results        <- read_csv("../results/K_place_fwd_results_2020-08-12.csv")
K_place_bkwd_results       <- read_csv("../results/K_place_bkwd_results_2020-08-12.csv")
K_major_place_fwd_results  <- read_csv("../results/K_major_place_fwd_results_2020-08-12.csv")
K_major_place_bkwd_results <- read_csv("../results/K_major_place_bkwd_results_2020-08-12.csv")
K_manner_fwd_results       <- read_csv("../results/K_manner_fwd_results_2020-08-12.csv")
K_manner_bkwd_results      <- read_csv("../results/K_manner_bkwd_results_2020-08-12.csv")

################################################################################
# Code used to produce summary table

place <- bind_rows(K_place_fwd_results, K_place_bkwd_results)
maj_place <- bind_rows(K_major_place_fwd_results, K_major_place_bkwd_results)
manner <- bind_rows(K_manner_fwd_results, K_manner_bkwd_results)

natural_classes_summary <- tibble("Classes" = c("Place", "Major place", "Manner"),
                                      "Mean K"  = c(mean(place$K), mean(maj_place$K), mean(manner$K)),
                                      "n characters" = c(nrow(place), nrow(maj_place), nrow(manner)),
                                      "significant (%)" = c(sum(place$significant == "yes"), sum(maj_place$significant == "yes"), sum(manner$significant == "yes")) 
)

kable(natural_classes_summary, digits = 2, format = "latex", booktabs = TRUE, caption = "Summary of $K$ analysis for forward and backward transition frequencies between different natural classes. The two rightmost columns indicate the total number of characters analyzed and the percentage of those characters with a significant degree of phylogenetic signal according to the randomization procedure.")

################################################################################
# K natural classes plot

k_place <- dplyr::select(place, K)
k_place$feature_type <- "Places of articulation"
k_major_place <- dplyr::select(maj_place, K)
k_major_place$feature_type <- "Major places of articulation"
k_manner <- dplyr::select(manner, K)
k_manner$feature_type <- "Manners of articulation"

k_all_classes <- bind_rows(k_place, k_major_place, k_manner)

K_natural_classes <- ggplot(k_all_classes, aes(K, fill = feature_type)) +
  geom_density(alpha = 0.5) +
  scale_fill_manual(values = c("Places of articulation" = "#d7191c",
                               "Major places of articulation" = "#66c2a5",
                               "Manners of articulation" = "#2c7bb6")) +
  labs(fill = "Natural class type") +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/k-natural-classes.pdf", width = 6.472, height = 4)
K_natural_classes
dev.off()

################################################################################
# K biphones vs classes

k_biphones <- dplyr::select(orig, K)
k_biphones$character_type <- "Biphones"
k_all_classes <- dplyr::select(k_all_classes, K)
k_all_classes$character_type <- "Natural classes"

all_k <- bind_rows(k_biphones, k_all_classes)

biphones_vs_classes <- ggplot(all_k, aes(K, fill = character_type)) +
  geom_density(alpha = 0.5) +
  scale_fill_manual(values = c("Natural classes" = "#d7191c", "Biphones" = "#2c7bb6")) +
  labs(fill = "Character type") +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/k-biphones-classes.pdf", width = 6.472, height = 4)
biphones_vs_classes
dev.off()

################################################################################
# K MCC tree vs posterior tree sample

load("../results/tree_uncertainty_2020-08-13.Rdata")

trees_place_fwd <- K_place_fwd_results
trees_place_bkwd <- K_place_bkwd_results
trees_manner_fwd <- K_manner_fwd_results
trees_manner_bkwd <- K_manner_bkwd_results

all_trees_K <- c(sapply(trees_place_fwd, `[[`, "K"),
                 sapply(trees_place_bkwd, `[[`, "K"),
                 sapply(trees_manner_fwd, `[[`, "K"),
                 sapply(trees_manner_bkwd, `[[`, "K"))

posterior <- tibble(K = all_trees_K, tree = "Posterior tree sample")
mcct <- tibble(K = c(place$K, manner$K), tree = "Max. clade credibility tree")
posterior_and_mcct <- bind_rows(posterior, mcct)

mcct_vs_posterior <- ggplot(posterior_and_mcct, aes(K, fill = tree)) +
  geom_density(alpha = 0.5) +
  scale_fill_manual(values = c("Max. clade credibility tree" = "#d7191c", "Posterior tree sample" = "#2c7bb6")) +
  labs(fill = "Reference tree") +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/k-mcct-posterior.pdf", width = 6.472, height = 4)
mcct_vs_posterior
dev.off()

################################################################################
# Wordlist subset sizes

IQR_wordlists <- filter(wordlists, between(wordlists$n_entries, quantile(wordlists$n_entries)[2], quantile(wordlists$n_entries)[4]))

IQR_wordlist_sizes <- ggplot(IQR_wordlists,
                       aes(x = reorder(tip_label, -n_entries), y = n_entries)) +
  geom_col(fill = "tomato3") +
  xlab("Language variety") +
  ylab("Wordlist size") +
  scale_y_continuous(expand = c(0, 0), limits = c(-25,5050)) +
  theme_classic(base_size = 10) +
  theme(plot.margin = margin(14,2,12,2,"pt"),
        axis.text.x = element_text(family = "Noto Sans",
                                   angle = 90, hjust=1, vjust=0.5, size = 6),
        axis.title.y = element_blank(),
        axis.title.x = element_blank())

Every2nd_wordlist <- arrange(wordlists, n_entries)[seq(2,111, by = 2),]

Every2nd_wordlist_sizes <- ggplot(Every2nd_wordlist,
                             aes(x = reorder(tip_label, -n_entries), y = n_entries)) +
  geom_col(fill = "tomato3") +
  xlab("Language variety") +
  ylab("Wordlist size") +
  scale_y_continuous(expand = c(0, 0), limits = c(-25,5050)) +
  theme_classic(base_size = 10) +
  theme(plot.margin = margin(14,2,12,2,"pt"),
        axis.text.x = element_text(family = "Noto Sans",
                                   angle = 90, hjust=1, vjust=0.5, size = 6),
        axis.title.x = element_blank())

cairo_pdf("../fig/wordlist_subset_sizes.pdf", width = 10, height = (5.5 * 0.618))
plot_grid(Every2nd_wordlist_sizes, IQR_wordlist_sizes, label_size = 10, hjust = 0, vjust = 1.2,
          labels = c("(A)", "(B)")) +
  draw_label("Language variety", vjust = 16.5, size = 10)
dev.off()

################################################################################
# K Middle 50% of wordlists vs every 2nd wordlist

IQR_manner_fwd <- read_csv("../results/IQR_K_manner_fwd_results_2020-08-12.csv")
IQR_manner_bkwd <- read_csv("../results/IQR_K_manner_bkwd_results_2020-08-12.csv")
IQR_place_fwd <- read_csv("../results/IQR_K_place_fwd_results_2020-08-12.csv")
IQR_place_bkwd <- read_csv("../results/IQR_K_place_bkwd_results_2020-08-12.csv")

Every2nd_manner_fwd <- read_csv("../results/Every2nd_K_manner_fwd_results_2020-08-12.csv")
Every2nd_manner_bkwd <- read_csv("../results/Every2nd_K_manner_bkwd_results_2020-08-12.csv")
Every2nd_place_fwd <- read_csv("../results/Every2nd_K_place_fwd_results_2020-08-12.csv")
Every2nd_place_bkwd <- read_csv("../results/Every2nd_K_place_bkwd_results_2020-08-12.csv")

IQR <- bind_rows(IQR_manner_fwd, IQR_manner_bkwd, IQR_place_fwd, IQR_place_bkwd)
IQR$wordlists <- "Middle 50% of wordlists"
Every2nd <- bind_rows(Every2nd_manner_fwd, Every2nd_manner_bkwd, Every2nd_place_fwd, Every2nd_place_bkwd)
Every2nd$wordlists <- "Every 2nd wordlist"

IQR_and_Every2nd <- bind_rows(IQR, Every2nd)

IQR_vs_Every2nd <- ggplot(IQR_and_Every2nd, aes(K, fill = wordlists)) +
  geom_density(alpha = 0.5) +
  scale_fill_manual(values = c("Middle 50% of wordlists" = "#d7191c", "Every 2nd wordlist" = "#2c7bb6")) +
  labs(fill = "Wordlists") +
  ylab("Density") +
  theme_cowplot()

cairo_pdf("../fig/k-IQR-vs-Every2nd.pdf", width = 6.472, height = 4)
IQR_vs_Every2nd
dev.off()

################################################################################
# Pama-Nyungan reference tree for Figure S1 (Supplementary Information)
# Tree loaded, pruned and saved using treeio package
# (which does a better job than ape at saving node labels etc)
# Then opened and appearance of labels etc adjusted manually in FigTree

# PN_beast <- treeio::read.beast("../trees/PNY10_285.(time).sum.tree")
# PN_beast_pruned <- treeio::drop.tip(PN_beast, setdiff(PN_beast@phylo$tip.label, wordlists$tip_label))
# treeio::write.beast(PN_beast_pruned, "../trees/PN_beast_pruned.tree")
