# Modified Caper Functions
# Jayden Macklin-Cordes
# 25 June 2018
# # # # # #

# This script provides two minimally modified versions of the functions 'phylo.d' and 'comparative.data' from the package Caper.
# The purpose of these modifications is to avoid undesirable behaviour by applying deparse(substitute(x)) to variable names. This means you have to give the function a variable name unquoted---regular character strings or named R variables don't work.
# Removing or minimally modifying these lines in the code allows one to provide a regular character string or named R object to each function.
# Original lines are retained but commented out, with modified line immediately below.

mod.phylo.d <-    function (data, phy, names.col, binvar, permut = 1000, rnd.bias = NULL) 
{
    if (!missing(data)) {
        if (!inherits(data, "comparative.data")) {
            if (missing(names.col)) 
                stop("names column is missing")
            names.col <- deparse(substitute(names.col))
            data <- caicStyleArgs(data = data, phy = phy, names.col = names.col)
        }
    }
#    binvar <- deparse(substitute(binvar))
    bininds <- match(binvar, names(data$data))
    if (is.na(bininds)) 
        (stop("'", binvar, "' is not a variable in data."))
    ds <- data$data[, bininds]
    if (any(is.na(ds))) 
        stop("'", binvar, "' contains missing values.")
    if (is.character(ds)) 
        ds <- as.factor(ds)
    if (length(unique(ds)) > 2) 
        stop("'", binvar, "' contains more than two states.")
    if (length(unique(ds)) < 2) 
        stop("'", binvar, "' only contains a single state.")
    propStates <- unclass(table(ds))
    propState1 <- propStates[1]/sum(propStates)
    names(dimnames(propStates)) <- binvar
    if (is.factor(ds)) 
        ds <- as.numeric(ds)
    if (!is.numeric(permut)) 
        (stop("'", permut, "' is not numeric."))
    if (!is.null(rnd.bias)) {
        rnd.bias <- deparse(substitute(rnd.bias))
        rnd.ind <- match(rnd.bias, names(data$data))
        if (is.na(rnd.ind)) 
            (stop("'", rnd.bias, "' is not a variable in data."))
        rnd.bias <- data$data[, rnd.bias]
    }
    el <- data$phy$edge.length
    elTip <- data$phy$edge[, 2] <= length(data$phy$tip.label)
    if (any(el[elTip] == 0)) 
        stop("Phylogeny contains pairs of tips on zero branch lengths, cannot currently simulate")
    if (any(el[!elTip] == 0)) 
        stop("Phylogeny contains zero length internal branches. Use di2multi.")
    ds.ran <- replicate(permut, sample(ds, prob = rnd.bias))
    if (is.null(data$vcv)) {
        vcv <- VCV.array(data$phy)
    }
    else {
        vcv <- data$vcv
    }
    ds.phy <- rmvnorm(permut, sigma = unclass(vcv))
    ds.phy <- as.data.frame(t(ds.phy))
    ds.phy.thresh <- apply(ds.phy, 2, quantile, propState1)
    ds.phy <- sweep(ds.phy, 2, ds.phy.thresh, "<")
    ds.phy <- as.numeric(ds.phy)
    dim(ds.phy) <- dim(ds.ran)
    ds.ran <- cbind(Obs = ds, ds.ran)
    ds.phy <- cbind(Obs = ds, ds.phy)
    dimnames(ds.ran) <- dimnames(ds.phy) <- list(data$phy$tip.label, 
        c("Obs", paste("V", 1:permut, sep = "")))
    phy <- reorder(data$phy, "pruningwise")
    ds.ran.cc <- contrCalc(vals = ds.ran, phy = phy, ref.var = "V1", 
        picMethod = "phylo.d", crunch.brlen = 0)
    ds.phy.cc <- contrCalc(vals = ds.phy, phy = phy, ref.var = "V1", 
        picMethod = "phylo.d", crunch.brlen = 0)
    ransocc <- colSums(ds.ran.cc$contrMat)
    physocc <- colSums(ds.phy.cc$contrMat)
    if (round(ransocc[1], digits = 6) != round(physocc[1], digits = 6)) 
        stop("Problem with character change calculation in phylo.d")
    obssocc <- ransocc[1]
    ransocc <- ransocc[-1]
    physocc <- physocc[-1]
    soccratio <- (obssocc - mean(physocc))/(mean(ransocc) - mean(physocc))
    soccpval1 <- sum(ransocc < obssocc)/permut
    soccpval0 <- sum(physocc > obssocc)/permut
    dvals <- list(DEstimate = soccratio, Pval1 = soccpval1, Pval0 = soccpval0, 
        Parameters = list(Observed = obssocc, MeanRandom = mean(ransocc), 
            MeanBrownian = mean(physocc)), StatesTable = propStates, 
        Permutations = list(random = ransocc, brownian = physocc), 
        NodalVals = list(observed = ds.ran.cc$nodVals[, 1, drop = FALSE], 
            random = ds.ran.cc$nodVals[, -1, drop = FALSE], brownian = ds.phy.cc$nodVals[, 
                -1, drop = FALSE]), binvar = binvar, data = data, 
        nPermut = permut, rnd.bias = rnd.bias)
    class(dvals) <- "phylo.d"
    return(dvals)
}

mod.comparative.data <- function (phy, data, names.col, vcv = FALSE, vcv.dim = 2, na.omit = TRUE, 
    force.root = FALSE, warn.dropped = FALSE, scope = NULL) 
{
    phy.name <- deparse(substitute(phy))
    data.name <- deparse(substitute(data))
#    names.col <- as.character(substitute(names.col))
     names.col <- names.col
    if (!is.data.frame(data)) 
        stop("'data' must be an object of class 'data.frame'.")
    namesInd <- match(names.col, names(data))
    if (is.na(namesInd)) {
        stop("Names column '", names.col, "' not found in data frame '", 
            data.name, "'")
    }
    rownames(data) <- as.character(data[, namesInd])
    data <- data[, -namesInd, drop = FALSE]
    if (!inherits(phy, "phylo")) 
        stop("'", deparse(substitute(phy)), "' not of class 'phylo'")
    if (!is.rooted(phy)) {
        if (force.root) {
            phy$root.edge <- 1
        }
        else {
            stop("'", deparse(substitute(phy)), "' is not rooted or has a basal polytomy.")
        }
    }
    if (any(duplicated(phy$tip.label))) 
        stop("Duplicate tip labels present in phylogeny")
    if (any(duplicated(c(phy$tip.label, phy$node.label)))) 
        stop("Labels duplicated between tips and nodes in phylogeny")
    origTips <- with(phy, max(edge) - Nnode)
    origData <- nrow(data)
    in.both <- intersect(rownames(data), phy$tip.label)
    if (length(in.both) == 0) 
        stop("No tips are common to the dataset and phylogeny")
    row.in.tree <- match(rownames(data), in.both)
    row.not.in.tree <- rownames(data)[is.na(row.in.tree)]
    data <- subset(data, !is.na(row.in.tree))
    tip.in.data <- match(phy$tip.label, in.both)
    to.drop <- phy$tip.label[is.na(tip.in.data)]
    if (length(to.drop) > 0) 
        matchedPhy <- drop.tip(phy, to.drop)
    else matchedPhy <- phy
    root <- length(matchedPhy$tip.label) + 1
    tip.order <- match(matchedPhy$tip.label, rownames(data))
    if (any(is.na(tip.order))) 
        stop("Problem with sorting data frame: mismatch between tip labels and data frame labels")
    data <- data[tip.order, , drop = FALSE]
    rownames(data) <- matchedPhy$tip.label
    IntNd <- root:max(matchedPhy$edge)
    if (is.null(matchedPhy$node.label)) {
        matchedPhy$node.label <- IntNd
    }
    else {
        matchedPhy$node.label <- ifelse(matchedPhy$node.label == 
            "", NA, matchedPhy$node.label)
        if (any(duplicated(na.omit(matchedPhy$node.label)))) 
            stop("Duplicate node labels present in phylogeny")
        matchedPhy$node.label <- ifelse(is.na(matchedPhy$node.label), 
            IntNd, matchedPhy$node.label)
    }
    RET <- list(phy = matchedPhy, data = data, data.name = data.name, 
        phy.name = phy.name, dropped = list(tips = to.drop, unmatched.rows = row.not.in.tree))
    class(RET) <- "comparative.data"
    if (vcv) {
        RET$vcv <- VCV.array(matchedPhy, dim = vcv.dim)
        RET$vcv.dim <- vcv.dim
    }
    if (na.omit) {
        before.drop.rows <- rownames(RET$data)
        RET <- na.omit(RET, scope)
        if (!identical(rownames(RET$data), before.drop.rows)) 
            RET$dropped$NA.rows <- before.drop.rows
    }
    if (warn.dropped) {
        if (any(sapply(RET$dropped, length) > 0)) 
            warning("Data dropped in compiling comparative data object")
    }
    return(RET)
}