# Phylogenetic signal with uncertainty in Pama-Nyungan reference tree
# Jayden Macklin-Cordes

##############################
# SET UP
##############################

# Load packages
library(tidyverse)
library(ape)
library(picante)
library(phangorn)
library(pbapply)

# Set seed for reproducibility of stochastic processes
set.seed(2019)

# Read posterior sample of reference trees
PN_SDollo <- read.nexus("../trees/PNY10_285.trees")

# Sample 100 best trees according to the maximum clade credibility metric
get_tree_sample = function(trees, n_trees = 100, burnin = 100) {
  trees <- trees[-(1:burnin)]
  clade_cred_score <- maxCladeCred(x = trees, tree = FALSE)
  clade_cred_rank <- rank(clade_cred_score * -1, ties.method = "first")
  ranked_top_n <- match(1:n_trees, clade_cred_rank)
  tree_sample <- trees[ranked_top_n]
  tree_sample
}

PN_sample <- get_tree_sample(PN_SDollo)

##############################
# PREPARE DATA
##############################

# Function to filter out characters with too many missing values
# The rm_0 parameter optionally converts 0 values to NAs (important for K test)
filter_phylo_data <- function(dataset, lim = 50, rm_0 = FALSE) {
  if (rm_0 == TRUE) {
    dataset[dataset == 0] <- NA
  }
  df <- select_if(dataset, ~sum(!is.na(.)) >= lim)
  df
}

# Function to drop uninformative variables (where all values are the same)
drop_uninformative_vars <- function(dataset) {
  uniquelength <- sapply(dataset, function(x) length(unique(x[!is.na(x)])))
  df <- subset(dataset, select = uniquelength > 1)
  df
}

# Prepare sound class frequency data (min 20 non-NA values)
K_place_fwd <- read_csv("../data/place_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_place_bkwd <- read_csv("../data/place_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_manner_fwd <- read_csv("../data/manner_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

K_manner_bkwd <- read_csv("../data/manner_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars()

##############################
# K TEST WITH NATURAL CLASSES
##############################

# Set up analysis
# Create data frame with rownames, as required by multiPhylosignal
create_char_df <- function(dat) {
  df <- as.data.frame(dat)
  rownames(df) <- df$tip_label
  df[, "tip_label"] <- NULL
  df
}

# Extra functions for tabulating results
one_tailed_p <- function (p, alpha = 0.05) {
  if (p < alpha) {
    sig <- "yes"
  } else {
    sig <- "no"
  }
  sig
}

extra_K_analysis <- function(results, dataset) {
  results$significant <- sapply(results$PIC.variance.P, function(x) as.factor(one_tailed_p(x)))
  results$biphone <- rownames(results)
  results$n_langs <- sapply(results$biphone, function(x) sum(!is.na(dataset[,x])))
  results
}

# Put steps together into single function
full_K_analysis <- function (dat, tree) {
  result <- dat %>% 
    create_char_df %>%
    multiPhylosignal(tree, reps = 10000) %>%
    extra_K_analysis(dat)
  result
}

# Perform analysis
K_place_fwd_results <- pblapply(1:length(PN_sample), function (i) full_K_analysis(K_place_fwd, PN_sample[[i]]))
K_place_bkwd_results <- pblapply(1:length(PN_sample), function (i) full_K_analysis(K_place_bkwd, PN_sample[[i]]))
K_manner_fwd_results <- pblapply(1:length(PN_sample), function (i) full_K_analysis(K_manner_fwd, PN_sample[[i]]))
K_manner_bkwd_results <- pblapply(1:length(PN_sample), function (i) full_K_analysis(K_manner_bkwd, PN_sample[[i]]))

save(K_place_fwd_results, K_place_bkwd_results, K_manner_fwd_results, K_manner_bkwd_results, 
     file = paste0("../results/tree_uncertainty_", Sys.Date(), ".Rdata"))
