##############################
# SET UP
##############################

# Load packages
library(tidyverse)
library(ape)
library(picante)

# Set seed for reproducibility of stochastic processes
set.seed(2019)

######################################
# K TEST WITH MIDDLE 50% OF WORDLISTS
######################################

# Get wordlist sizes
lex_sizes <- read_csv("../data/wordlist_sizes_2020-08-07.csv")
lex_IQR <- filter(lex_sizes, between(lex_sizes$n_entries, quantile(lex_sizes$n_entries)[2], quantile(lex_sizes$n_entries)[4]))

# Read reference tree
PN_SDollo_sum <- read.nexus("../trees/PNY10_285.(time).sum.tree")

### PREPARE DATA ###

# Function to filter out characters with too many missing values
# The rm_0 parameter optionally converts 0 values to NAs (important for K test)
filter_phylo_data <- function(dataset, lim = 50, rm_0 = FALSE) {
  if (rm_0 == TRUE) {
    dataset[dataset == 0] <- NA
  }
  df <- select_if(dataset, ~sum(!is.na(.)) >= lim)
  df
}

# Function to drop uninformative variables (where all values are the same)
drop_uninformative_vars <- function(dataset) {
  uniquelength <- sapply(dataset, function(x) length(unique(x[!is.na(x)])))
  df <- subset(dataset, select = uniquelength > 1)
  df
}

# Prepare sound class frequency data (min 20 non-NA values)
K_place_fwd <- read_csv("../data/place_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_IQR$tip_label)

K_place_bkwd <- read_csv("../data/place_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_IQR$tip_label)

K_manner_fwd <- read_csv("../data/manner_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_IQR$tip_label)

K_manner_bkwd <- read_csv("../data/manner_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_IQR$tip_label)

### ANALYSIS ###

# Create data frame with rownames, as required by multiPhylosignal
create_char_df <- function(dat) {
  df <- as.data.frame(dat)
  rownames(df) <- df$tip_label
  df[, "tip_label"] <- NULL
  df
}

# Extra functions for tabulating results
one_tailed_p <- function (p, alpha = 0.05) {
  if (p < alpha) {
    sig <- "yes"
  } else {
    sig <- "no"
  }
  sig
}

extra_K_analysis <- function(results, dataset) {
  results$significant <- sapply(results$PIC.variance.P, function(x) as.factor(one_tailed_p(x)))
  results$biphone <- rownames(results)
  results$n_langs <- sapply(results$biphone, function(x) sum(!is.na(dataset[,x])))
  results
}

# Run test
K_place_fwd_results <- K_place_fwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_place_fwd)
K_place_bkwd_results <- K_place_bkwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_place_bkwd)
K_manner_fwd_results <- K_manner_fwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_manner_fwd)
K_manner_bkwd_results <- K_manner_bkwd %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_manner_bkwd)

write_csv(K_place_fwd_results, paste0("../results/IQR_K_place_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_place_bkwd_results, paste0("../results/IQR_K_place_bkwd_results_", Sys.Date(), ".csv"))
write_csv(K_manner_fwd_results, paste0("../results/IQR_K_manner_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_manner_bkwd_results, paste0("../results/IQR_K_manner_bkwd_results_", Sys.Date(), ".csv"))

#################################
# K TEST WITH EVERY 2ND WORDLIST
#################################

lex_every2nd <-  arrange(lex_sizes, n_entries)[seq(1,nrow(lex_sizes), by = 2),]

# Prepare sound class frequency data (min 20 non-NA values)
K_place_fwd_2        <- read_csv("../data/place_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_every2nd$tip_label)

K_place_bkwd_2       <- read_csv("../data/place_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_every2nd$tip_label)

K_manner_fwd_2       <- read_csv("../data/manner_fwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_every2nd$tip_label)

K_manner_bkwd_2      <- read_csv("../data/manner_bkwd_2020-08-07.csv") %>%
  filter_phylo_data(lim = 20, rm_0 = TRUE) %>%
  drop_uninformative_vars() %>%
  filter(tip_label %in% lex_every2nd$tip_label)

# Run test
K_place_fwd_2_results <- K_place_fwd_2 %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_place_fwd_2)
K_place_bkwd_2_results <- K_place_bkwd_2 %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_place_bkwd_2)
K_manner_fwd_2_results <- K_manner_fwd_2 %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_manner_fwd_2)
K_manner_bkwd_2_results <- K_manner_bkwd_2 %>%
  create_char_df %>%
  multiPhylosignal(PN_SDollo_sum, reps = 10000) %>%
  extra_K_analysis(K_manner_bkwd_2)

write_csv(K_place_fwd_2_results, paste0("../results/Every2nd_K_place_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_place_bkwd_2_results, paste0("../results/Every2nd_K_place_bkwd_results_", Sys.Date(), ".csv"))
write_csv(K_manner_fwd_2_results, paste0("../results/Every2nd_K_manner_fwd_results_", Sys.Date(), ".csv"))
write_csv(K_manner_bkwd_2_results, paste0("../results/Every2nd_K_manner_bkwd_results_", Sys.Date(), ".csv"))
